# Simple linear regression class
# Note: This program is originally implemented in C++ by David C. Swaim II
# URL: http://www.oocities.org/david_swaim/cpp/linreg.htm

import math

class Point2D:
    def __init__(self, x, y):
        self.x = x
        self.y = y
        
class SimpleLinearRegression:
    __sumX = 0.0
    __sumY = 0.0
    __sumXsquared = 0.0
    __sumYsquared = 0.0
    __sumXY = 0.0
    
    __a = 0.0
    __b = 0.0
    __coefD = 0.0
    __coefC = 0.0
    __stdError = 0.0
    __meanSquaredError = 0.0
    
    def __init__(self, point2DList, size):
        self.point2DList = point2DList
        self.size = size
        for point in self.point2DList:
            self.__sumX += point.x
            self.__sumY += point.y
            self.__sumXsquared += point.x * point.x
            self.__sumYsquared += point.y * point.y
            self.__sumXY += point.x * point.y
      
    def calculate(self):
        if math.fabs(self.size * self.__sumXsquared - self.__sumX * self.__sumX) > 0.00000001:
            self.__b = (self.size * self.__sumXY - self.__sumY * self.__sumX) / (self.size * self.__sumXsquared - self.__sumX * self.__sumX)
            self.__a = (self.__sumY - self.__b * self.__sumX) / self.size
            sx = self.__b * (self.__sumXY - self.__sumX * self.__sumY / self.size)
            sy2 = self.__sumYsquared - self.__sumY * self.__sumY / self.size
            sy = sy2 - sx
            
            self.__coefD = sx / sy2
            self.__coefC = math.sqrt(self.__coefD)
            self.__stdError = math.sqrt(sy / (self.size - 2))
            
            sumSquaredError = 0.0
            for point in self.point2DList:
                sumSquaredError += (point.y - (self.__a + self.__b * point.x)) * (point.y - (self.__a + self.__b * point.x))
            self.__meanSquaredError = sumSquaredError / self.size
        else:
            self.__a = self.__b = self.__coefD = self.__coefC = self.__stdError = 0
  
    def get_a(self):
        return self.__a
           
    def get_b(self):
        return self.__b
           
    def get_coefD(self):
        return self.__coefD
           
    def get_coefC(self):
        return self.__coefC
            
    def get_stdError(self):
        return self.__stdError
        
    def get_meanSquaredError(self):
        return self.__meanSquaredError
    
    a = property(get_a)
    b = property(get_b)
    coefD = property(get_coefD)  
    coefC = property(get_coefC)
    stdError = property(get_stdError)
    meanSquaredError = property(get_meanSquaredError)

        
